@@ -399,9 +399,13 @@ def is_pinned_requirement(ireq):
 
 
 def is_editable_path(path):
-    if os.path.isdir(path):
-        return True
-    return False
+    """
+    Determine if a path is editable by checking if it's a directory.
+
+    :param path: A path as string or Path object
+    :return: True if the path is a directory, False otherwise
+    """
+    return Path(path).is_dir()
 
 
 def dependency_as_pip_install_line(
@@ -662,35 +666,44 @@ def find_package_name_from_zipfile(zip_filepath):
 
 def find_package_name_from_directory(directory):
     parsed_url = urlparse(directory)
-    directory = (
-        os.path.normpath(parsed_url.path)
-        if parsed_url.scheme
-        else os.path.normpath(directory)
-    )
-    if "#egg=" in directory:  # parse includes the fragment in py3.7 and py3.8
-        expected_name = directory.split("#egg=")[1]
+    directory_path = Path(parsed_url.path) if parsed_url.scheme else Path(directory)
+
+    # Handle egg fragment for direct dependencies
+    if "#egg=" in str(directory_path):
+        expected_name = str(directory_path).split("#egg=")[1]
         return expected_name
+
+    # Windows path normalization
+    directory_str = str(directory_path)
     if os.name == "nt":
-        if directory.startswith("\\") and (":\\" in directory or ":/" in directory):
-            directory = directory[1:]
-        if directory.startswith("\\\\"):
-            directory = directory[1:]
-    directory_contents = sorted(
-        os.listdir(directory),
-        key=lambda x: (os.path.isdir(os.path.join(directory, x)), x),
-    )
-    for filename in directory_contents:
-        filepath = os.path.join(directory, filename)
-        if os.path.isfile(filepath):
-            if filename.endswith(RELEVANT_PROJECT_FILES):
-                with open(filepath, "rb") as file:
-                    possible_name = find_package_name_from_filename(filename, file)
-                    if possible_name:
-                        return possible_name
-        elif os.path.isdir(filepath):
-            possible_name = find_package_name_from_directory(filepath)
-            if possible_name:
-                return possible_name
+        if directory_str.startswith("\\") and (
+            ":\\" in directory_str or ":/" in directory_str
+        ):
+            directory_path = Path(directory_str[1:])
+        if directory_str.startswith("\\\\"):
+            directory_path = Path(directory_str[1:])
+
+    try:
+        # Sort contents - files first, then directories to search parent
+        # directories before leaf directories.
+        directory_contents = sorted(
+            directory_path.iterdir(), key=lambda x: (x.is_dir(), x.name)
+        )
+
+        for path in directory_contents:
+            if path.is_file():
+                if path.name.endswith(RELEVANT_PROJECT_FILES):
+                    with path.open("rb") as file:
+                        possible_name = find_package_name_from_filename(path.name, file)
+                        if possible_name:
+                            return possible_name
+            elif path.is_dir():
+                possible_name = find_package_name_from_directory(str(path))
+                if possible_name:
+                    return possible_name
+    except (FileNotFoundError, PermissionError):
+        # Handle cases where the directory doesn't exist or isn't accessible
+        pass
 
     return None
 
@@ -788,7 +801,7 @@ def determine_package_name(package: InstallRequirement):
                 local_file = unpack_url(
                     link=package.link,
                     location=td,
-                    download=Downloader(session, "off"),
+                    download=Downloader(session, "off", resume_retries=5),
                     verbosity=1,
                 )
                 if local_file.path.endswith(".whl") or local_file.path.endswith(".zip"):
@@ -832,31 +845,34 @@ def determine_package_name(package: InstallRequirement):
 
 
 def find_package_name_from_filename(filename, file):
-    if filename.endswith("METADATA"):
+    # Extract basename to handle both full paths (from archives) and basenames (from directories)
+    basename = Path(filename).name
+
+    if basename == "METADATA":
         content = file.read().decode()
         possible_name = parse_metadata_file(content)
         if possible_name:
             return possible_name
 
-    if filename.endswith("PKG-INFO"):
+    if basename == "PKG-INFO":
         content = file.read().decode()
         possible_name = parse_pkginfo_file(content)
         if possible_name:
             return possible_name
 
-    if filename.endswith("setup.py"):
+    if basename == "setup.py":
         content = file.read().decode()
         possible_name = parse_setup_file(content)
         if possible_name:
             return possible_name
 
-    if filename.endswith("setup.cfg"):
+    if basename == "setup.cfg":
         content = file.read().decode()
         possible_name = parse_cfg_file(content)
         if possible_name:
             return possible_name
 
-    if filename.endswith("pyproject.toml"):
+    if basename == "pyproject.toml":
         content = file.read().decode()
         possible_name = parse_toml_file(content)
         if possible_name:
@@ -987,6 +1003,11 @@ def expansive_install_req_from_line(
     """
     name = None
     pip_line = pip_line.strip("'").lstrip(" ")
+
+    # Handle paths with escaped spaces
+    if os.path.exists(os.path.expanduser(pip_line.replace("\\ ", " "))):
+        pip_line = pip_line.replace("\\ ", " ")
+
     for new_req_symbol in ("@ ", " @ "):  # Check for new style pip lines
         if new_req_symbol in pip_line:
             pip_line_parts = pip_line.split(new_req_symbol, 1)
@@ -1047,6 +1068,13 @@ def expansive_install_req_from_line(
     return install_req, name
 
 
+def normalize_editable_path_for_pip(path_str):
+    """Normalize an editable package path for pip."""
+    # Windows paths need to be converted to POSIX paths otherwise path
+    # separators (back slashes) are interpreted as escape characters.
+    return path_str.replace(os.path.sep, "/")
+
+
 def file_path_from_pipfile(path_str, pipfile_entry):
     """Creates an installable file path from a pipfile entry.
     Handles local and remote paths, files and directories;
@@ -1061,7 +1089,7 @@ def file_path_from_pipfile(path_str, pipfile_entry):
     if pipfile_entry.get("extras"):
         req_str = f"{req_str}[{','.join(pipfile_entry['extras'])}]"
     if pipfile_entry.get("editable", False):
-        req_str = f"-e {req_str}"
+        req_str = f"-e {normalize_editable_path_for_pip(req_str)}"
 
     return req_str
 
@@ -1261,17 +1289,16 @@ def get_constraints_from_deps(deps):
                 c = f"{canonicalize_name(dep_name)}{dep_version}"
             else:
                 c = canonicalize_name(dep_name)
-        else:
-            if not any(k in dep_version for k in ["path", "file", "uri"]):
-                if dep_version.get("skip_resolver") is True:
-                    continue
-                version = dep_version.get("version", None)
-                if version and not is_star(version):
-                    if COMPARE_OP.match(version) is None:
-                        version = f"=={dep_version}"
-                    c = f"{canonicalize_name(dep_name)}{version}"
-                else:
-                    c = canonicalize_name(dep_name)
+        elif not any(k in dep_version for k in ["path", "file", "uri"]):
+            if dep_version.get("skip_resolver") is True:
+                continue
+            version = dep_version.get("version", None)
+            if version and not is_star(version):
+                if COMPARE_OP.match(version) is None:
+                    version = f"=={dep_version}"
+                c = f"{canonicalize_name(dep_name)}{version}"
+            else:
+                c = canonicalize_name(dep_name)
         if c:
             constraints.add(c)
     return constraints
